@@ -33,6 +33,7 @@
 #define DP_TMPFSUF     MYEXTSTR "dptmp"  /* suffix of a temporary file */
 #define DP_OPTBLOAD    0.25              /* ratio of bucket loading at optimization */
 #define DP_OPTRUNIT    256               /* number of records in a process of optimization */
+#define DP_NUMBUFSIZ   32                /* size of a buffer for a number */
 #define DP_IOBUFSIZ    8192              /* size of an I/O buffer */
 
 enum {                                   /* enumeration for a record header */
@@ -1032,6 +1033,132 @@ int dprepair(const char *name){
 }
 
 
+/* Dump all records as endian independent data. */
+int dpexportdb(DEPOT *depot, const char *name){
+  char *kbuf, *vbuf, *pbuf;
+  int fd, err, ksiz, vsiz, psiz;
+  assert(depot && name);
+  if(!(dpiterinit(depot))) return FALSE;
+  if((fd = open(name, O_RDWR | O_CREAT | O_TRUNC, DP_FILEMODE)) == -1){
+    dpecodeset(DP_EOPEN, __FILE__, __LINE__);
+    return FALSE;
+  }
+  err = FALSE;
+  while(!err && (kbuf = dpiternext(depot, &ksiz)) != NULL){
+    if((vbuf = dpget(depot, kbuf, ksiz, 0, -1, &vsiz)) != NULL){
+      if((pbuf = malloc(ksiz + vsiz + DP_NUMBUFSIZ * 2)) != NULL){
+        psiz = 0;
+        psiz += sprintf(pbuf + psiz, "%X\n%X\n", ksiz, vsiz);
+        memcpy(pbuf + psiz, kbuf, ksiz);
+        psiz += ksiz;
+        pbuf[psiz++] = '\n';
+        memcpy(pbuf + psiz, vbuf, vsiz);
+        psiz += vsiz;
+        pbuf[psiz++] = '\n';
+        if(!dpwrite(fd, pbuf, psiz)){
+          dpecodeset(DP_EWRITE, __FILE__, __LINE__);
+          err = TRUE;
+        }
+        free(pbuf);
+      } else {
+        dpecodeset(DP_EALLOC, __FILE__, __LINE__);
+        err = TRUE;
+      }
+      free(vbuf);
+    } else {
+      err = TRUE;
+    }
+    free(kbuf);
+  }
+  if(close(fd) == -1){
+    if(!err) dpecodeset(DP_ECLOSE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  return !err && !dpfatalerror(depot);
+}
+
+
+/* Load all records from endian independent data. */
+int dpimportdb(DEPOT *depot, const char *name){
+  char mbuf[DP_IOBUFSIZ], *rbuf;
+  int i, j, fd, err, fsiz, off, msiz, ksiz, vsiz, hlen;
+  struct stat sbuf;
+  assert(depot && name);
+  if(!depot->wmode){
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if(dprnum(depot) > 0){
+    dpecodeset(DP_EMISC, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if((fd = open(name, O_RDONLY, DP_FILEMODE)) == -1){
+    dpecodeset(DP_EOPEN, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if(fstat(fd, &sbuf) == -1 || !S_ISREG(sbuf.st_mode)){
+    dpecodeset(DP_ESTAT, __FILE__, __LINE__);
+    close(fd);
+    return FALSE;
+  }
+  err = FALSE;
+  fsiz = sbuf.st_size;
+  off = 0;
+  while(!err && off < fsiz){
+    msiz = fsiz - off;
+    if(msiz > DP_IOBUFSIZ) msiz = DP_IOBUFSIZ;
+    if(!dpseekread(fd, off, mbuf, msiz)){
+      err = TRUE;
+      break;
+    }
+    hlen = 0;
+    ksiz = -1;
+    vsiz = -1;
+    for(i = 0; i < msiz; i++){
+      if(mbuf[i] == '\n'){
+        mbuf[i] = '\0';
+        ksiz = strtol(mbuf, NULL, 16);
+        for(j = i + 1; j < msiz; j++){
+          if(mbuf[j] == '\n'){
+            mbuf[j] = '\0';
+            vsiz = strtol(mbuf + i + 1, NULL, 16);
+            hlen = j + 1;
+            break;
+          }
+        }
+        break;
+      }
+    }
+    if(ksiz < 0 || vsiz < 0 || hlen < 4){
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+      err = TRUE;
+      break;
+    }
+    if(hlen + ksiz + vsiz + 2 < DP_IOBUFSIZ){
+      if(!dpput(depot, mbuf + hlen, ksiz, mbuf + hlen + ksiz + 1, vsiz, DP_DKEEP)) err = TRUE;
+    } else {
+      if((rbuf = malloc(ksiz + vsiz + 2)) != NULL){
+        if(dpseekread(fd, off + hlen, rbuf, ksiz + vsiz + 2)){
+          if(!dpput(depot, rbuf, ksiz, rbuf + ksiz + 1, vsiz, DP_DKEEP)) err = TRUE;
+        } else {
+          err = TRUE;
+        }
+        free(rbuf);
+      } else {
+        dpecodeset(DP_EALLOC, __FILE__, __LINE__);
+        err = TRUE;
+      }
+    }
+    off += hlen + ksiz + vsiz + 2;
+  }
+  if(close(fd) == -1){
+    if(!err) dpecodeset(DP_ECLOSE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  return !err && !dpfatalerror(depot);
+}
+
+
 /* Hash function used inside Depot. */
 int dpinnerhash(const char *kbuf, int ksiz){
   assert(kbuf);
