@@ -860,6 +860,7 @@ ARTICLE *makeartfromwiki(const char *wiki){
   const char *subtpcstr = "!!";
   const char *itemstr = "*";
   const char *asisstr = "|";
+  const char *asissecstr = "===|";
   const char *graphstr = "@graph:";
   const char *breakstr = "----";
   ARTICLE *art;
@@ -868,7 +869,7 @@ ARTICLE *makeartfromwiki(const char *wiki){
   const char *id, *password, *flags, *language, *category, *subject, *author, *cdate, *mdate;
   const char *delim, *cline, *pline, *nline;
   char *rtext;
-  int i, bs, be;
+  int i, bs, be, asis;
   assert(wiki);
   rtext = strnormalize(wiki, FALSE, FALSE);
   buf = cbdatumopen("", 0);
@@ -924,6 +925,7 @@ ARTICLE *makeartfromwiki(const char *wiki){
   xsprintf(buf, "<mdate>%@</mdate>\n", mdate);
   xsprintf(buf, "</head>\n");
   xsprintf(buf, "<body>\n");
+  asis = FALSE;
   be = cblistnum(lines);
   for(i = bs; i < cblistnum(lines); i++){
     if(!strcmp(cblistval(lines, i, NULL), delim)){
@@ -936,7 +938,14 @@ ARTICLE *makeartfromwiki(const char *wiki){
     cline = cblistval(lines, i, NULL);
     nline = i < be - 1 ? cblistval(lines, i + 1, NULL) : "";
     if(cline[0] != '\0'){
-      if(cbstrfwmatch(cline, subtpcstr)){
+      if(asis){
+        if(!strcmp(cline, asissecstr)){
+          xsprintf(buf, "</asis>\n");
+          asis = FALSE;
+        } else {
+          xsprintf(buf, "%@\n", cline);
+        }
+      } else if(cbstrfwmatch(cline, subtpcstr)){
         xsprintf(buf, "<subtopic>");
         wikicat(buf, cline + strlen(subtpcstr), TRUE);
         xsprintf(buf, "</subtopic>\n");
@@ -952,8 +961,11 @@ ARTICLE *makeartfromwiki(const char *wiki){
         if(!cbstrfwmatch(nline, itemstr)) xsprintf(buf, "</list>\n");
       } else if(cbstrfwmatch(cline, asisstr)){
         if(!cbstrfwmatch(pline, asisstr)) xsprintf(buf, "<asis>");
-        xsprintf(buf, "%s\n", cline + strlen(asisstr));
+        xsprintf(buf, "%@\n", cline + strlen(asisstr));
         if(!cbstrfwmatch(nline, asisstr)) xsprintf(buf, "</asis>\n");
+      } else if(!strcmp(cline, asissecstr)){
+        xsprintf(buf, "<asis>");
+        asis = TRUE;
       } else if(cbstrfwmatch(cline, graphstr)){
         cline += strlen(graphstr);
         while(*cline == ' '){
@@ -969,6 +981,7 @@ ARTICLE *makeartfromwiki(const char *wiki){
       }
     }
   }
+  if(asis) xsprintf(buf, "</asis>\n");
   xsprintf(buf, "</body>\n");
   xsprintf(buf, "<tail>\n");
   for(i = be + 1; i < cblistnum(lines) - 3; i += 4){
