@@ -21,9 +21,11 @@
 
 enum {                                   /* enumeration for the format of output data */
   FTXML,                                 /* original XML */
+  FTWIKI,                                /* Wiki-style text */
   FTTEXT,                                /* plain text */
   FTHTML,                                /* human readable HTML */
-  FTATOM                                 /* Atom feed */
+  FTATOM,                                /* Atom feed */
+  FTLATEX                                /* LaTeX text */
 };
 
 
@@ -45,18 +47,27 @@ int runexport(int argc, char **argv);
 int rundelete(int argc, char **argv);
 int runlist(int argc, char **argv);
 int rundump(int argc, char **argv);
+int runsavefile(int argc, char **argv);
+int runloadfile(int argc, char **argv);
+int rundeletefile(int argc, char **argv);
+int runlistfile(int argc, char **argv);
 int runoptimize(int argc, char **argv);
 int runinitialize(int argc, char **argv);
 int runpassword(int argc, char **argv);
-int procimport(const char *name, const char *file, const char *id);
+int procimport(const char *name, const char *file, const char *id, int ftype);
 int procexport(const char *name, const char *id, int ftype);
 int procdelete(const char *name, const char *id);
 int proclist(const char *name, int sort);
 int procdump(const char *name, int ftype);
+int procsavefile(const char *name, const char *file, const char *id, const char *password,
+                 const char *subject, const char *author, const char *cdate);
+int procloadfile(const char *name, const char *id, int al);
+int procdeletefile(const char *name, const char *id);
+int proclistfile(const char *name);
 int procoptimize(const char *name);
 int procinitialize(const char *name);
 int procpassword(const char *key, const char *code);
-char *readfiledata(const char *file);
+char *readfiledata(const char *file, int *sp);
 void xprintf(const char *format, ...);
 
 
@@ -78,6 +89,14 @@ int main(int argc, char **argv){
     rv = runlist(argc, argv);
   } else if(!strcmp(argv[1], "dump")){
     rv = rundump(argc, argv);
+  } else if(!strcmp(argv[1], "savefile")){
+    rv = runsavefile(argc, argv);
+  } else if(!strcmp(argv[1], "loadfile")){
+    rv = runloadfile(argc, argv);
+  } else if(!strcmp(argv[1], "deletefile")){
+    rv = rundeletefile(argc, argv);
+  } else if(!strcmp(argv[1], "listfile")){
+    rv = runlistfile(argc, argv);
   } else if(!strcmp(argv[1], "optimize")){
     rv = runoptimize(argc, argv);
   } else if(!strcmp(argv[1], "initialize")){
@@ -113,11 +132,16 @@ void usage(void){
   fprintf(stderr, "%s: administration utility for RBBS\n", g_progname);
   fprintf(stderr, "\n");
   fprintf(stderr, "usage:\n");
-  fprintf(stderr, "  %s import [-mod id] name [file]\n", g_progname);
-  fprintf(stderr, "  %s export [-ft] [-fh] [-fa] name key\n", g_progname);
-  fprintf(stderr, "  %s delete name key\n", g_progname);
+  fprintf(stderr, "  %s import [-mod id] [-fw] name [file]\n", g_progname);
+  fprintf(stderr, "  %s export [-ft] [-fh] [-fa] [-fl] [-fw] name id\n", g_progname);
+  fprintf(stderr, "  %s delete name id\n", g_progname);
   fprintf(stderr, "  %s list [-scd] [-sma] [-smd] name\n", g_progname);
-  fprintf(stderr, "  %s dump [-ft] [-fh] [-fa] name\n", g_progname);
+  fprintf(stderr, "  %s dump [-ft] [-fh] [-fa] [-fl] name\n", g_progname);
+  fprintf(stderr, "  %s savefile [-mod id] [-pw str] [-sj str] [-at str] [-cd str] name [file]\n",
+          g_progname);
+  fprintf(stderr, "  %s loadfile [-al] name id\n", g_progname);
+  fprintf(stderr, "  %s deletefile name id\n", g_progname);
+  fprintf(stderr, "  %s listfile name\n", g_progname);
   fprintf(stderr, "  %s optimize name\n", g_progname);
   fprintf(stderr, "  %s initialize name\n", g_progname);
   fprintf(stderr, "  %s password key [code]\n", g_progname);
@@ -129,15 +153,18 @@ void usage(void){
 /* parse arguments of import command */
 int runimport(int argc, char **argv){
   char *name, *file, *id;
-  int i, rv;
+  int i, ftype, rv;
   name = NULL;
   file = NULL;
   id = NULL;
+  ftype = FTXML;
   for(i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-mod")){
         if(++i >= argc) usage();
         id = argv[i];
+      } else if(!strcmp(argv[i], "-fw")){
+        ftype = FTWIKI;
       } else {
         usage();
       }
@@ -150,7 +177,7 @@ int runimport(int argc, char **argv){
     }
   }
   if(!name) usage();
-  rv = procimport(name, file, id);
+  rv = procimport(name, file, id, ftype);
   return rv;
 }
 
@@ -170,6 +197,10 @@ int runexport(int argc, char **argv){
         ftype = FTHTML;
       } else if(!strcmp(argv[i], "-fa")){
         ftype = FTATOM;
+      } else if(!strcmp(argv[i], "-fl")){
+        ftype = FTLATEX;
+      } else if(!strcmp(argv[i], "-fw")){
+        ftype = FTWIKI;
       } else {
         usage();
       }
@@ -253,6 +284,8 @@ int rundump(int argc, char **argv){
         ftype = FTHTML;
       } else if(!strcmp(argv[i], "-fa")){
         ftype = FTATOM;
+      } else if(!strcmp(argv[i], "-fl")){
+        ftype = FTLATEX;
       } else {
         usage();
       }
@@ -268,6 +301,130 @@ int rundump(int argc, char **argv){
 }
 
 
+/* parse arguments of savefile command */
+int runsavefile(int argc, char **argv){
+  char *name, *file, *id, *password, *subject, *author, *cdate, *date, *tmp;
+  int i, rv;
+  name = NULL;
+  file = NULL;
+  id = NULL;
+  password = NULL;
+  subject = NULL;
+  author = NULL;
+  cdate = NULL;
+  for(i = 2; i < argc; i++){
+    if(!name && argv[i][0] == '-'){
+      if(!strcmp(argv[i], "-mod")){
+        if(++i >= argc) usage();
+        id = argv[i];
+      } else if(!strcmp(argv[i], "-pw")){
+        if(++i >= argc) usage();
+        password = argv[i];
+      } else if(!strcmp(argv[i], "-sj")){
+        if(++i >= argc) usage();
+        subject = argv[i];
+      } else if(!strcmp(argv[i], "-at")){
+        if(++i >= argc) usage();
+        author = argv[i];
+      } else if(!strcmp(argv[i], "-cd")){
+        if(++i >= argc) usage();
+        cdate = argv[i];
+      } else {
+        usage();
+      }
+    } else if(!name){
+      name = argv[i];
+    } else if(!file){
+      file = argv[i];
+    } else {
+      usage();
+    }
+  }
+  if(!name) usage();
+  date = datecurrent();
+  if(!password) password = "";
+  if(!subject) subject = file ? file : "";
+  if((tmp = strrchr(subject, '\\')) != NULL) subject = tmp + 1;
+  if((tmp = strrchr(subject, '/')) != NULL) subject = tmp + 1;
+  if(!author) author = "";
+  if(!cdate) cdate = date;
+  rv = procsavefile(name, file, id, password, subject, author, cdate);
+  free(date);
+  return rv;
+}
+
+
+/* parse arguments of loadfile command */
+int runloadfile(int argc, char **argv){
+  char *name, *id;
+  int i, rv, al;
+  name = NULL;
+  id = NULL;
+  al = FALSE;
+  for(i = 2; i < argc; i++){
+    if(!name && argv[i][0] == '-'){
+      if(!strcmp(argv[i], "-al")){
+        al = TRUE;
+      } else {
+        usage();
+      }
+    } else if(!name){
+      name = argv[i];
+    } else if(!id){
+      id = argv[i];
+    } else {
+      usage();
+    }
+  }
+  if(!name || !id) usage();
+  rv = procloadfile(name, id, al);
+  return rv;
+}
+
+
+/* parse arguments of deletefile command */
+int rundeletefile(int argc, char **argv){
+  char *name, *id;
+  int i, rv;
+  name = NULL;
+  id = NULL;
+  for(i = 2; i < argc; i++){
+    if(!name && argv[i][0] == '-'){
+      usage();
+    } else if(!name){
+      name = argv[i];
+    } else if(!id){
+      id = argv[i];
+    } else {
+      usage();
+    }
+  }
+  if(!name || !id) usage();
+  rv = procdeletefile(name, id);
+  return rv;
+}
+
+
+/* parse arguments of listfile command */
+int runlistfile(int argc, char **argv){
+  char *name;
+  int i, rv;
+  name = NULL;
+  for(i = 2; i < argc; i++){
+    if(!name && argv[i][0] == '-'){
+      usage();
+    } else if(!name){
+      name = argv[i];
+    } else {
+      usage();
+    }
+  }
+  if(!name) usage();
+  rv = proclistfile(name);
+  return rv;
+}
+
+
 /* parse arguments of optimize command */
 int runoptimize(int argc, char **argv){
   char *name;
@@ -330,7 +487,7 @@ int runpassword(int argc, char **argv){
 
 
 /* import an article */
-int procimport(const char *name, const char *file, const char *id){
+int procimport(const char *name, const char *file, const char *id, int ftype){
   RBBS *rbbs;
   ARTICLE *art;
   char *xml;
@@ -341,14 +498,21 @@ int procimport(const char *name, const char *file, const char *id){
   if(cbfilestat(name, &isdir, NULL, NULL) && !isdir){
     fprintf(stderr, "%s: %s: not a database\n", g_progname, name);
     rv = 1;
-  } else if(!(xml = readfiledata(file))){
+  } else if(!(xml = readfiledata(file, NULL))){
     fprintf(stderr, "%s: %s: cannot read\n", g_progname, file ? file : "(stdin)");
     rv = 1;
   } else if(!(rbbs = rbbsopen(name, TRUE))){
     fprintf(stderr, "%s: %s: cannot open\n", g_progname, name);
     rv = 1;
   } else {
-    art = makearticle(xml);
+    switch(ftype){
+    case FTWIKI:
+      art = makeartfromwiki(xml);
+      break;
+    default:
+      art = makearticle(xml);
+      break;
+    }
     if(!articleisok(art)){
       fprintf(stderr, "%s: %s\n", g_progname, artemsg(art));
       rv = 1;
@@ -404,6 +568,20 @@ int procexport(const char *name, const char *id, int ftype){
       printf("%s", xml);
       free(xml);
       break;
+    case FTLATEX:
+      printf("%s\n", latexpreamble("article", artsubject(art), artauthor(art), artmdate(art)));
+      printf("\\begin{document}\n\n");
+      printf("\\maketitle\n\n");
+      xml = arttolatex(art, FALSE);
+      printf("%s", xml);
+      free(xml);
+      printf("\\end{document}\n");
+      break;
+    case FTWIKI:
+      xml = arttowiki(art);
+      printf("%s", xml);
+      free(xml);
+      break;
     default:
       xml = arttoxml(art);
       printf("%s", xml);
@@ -423,11 +601,9 @@ int procexport(const char *name, const char *id, int ftype){
 /* delete an article */
 int procdelete(const char *name, const char *id){
   RBBS *rbbs;
-  ARTICLE *art;
   int rv;
   rv = 0;
   rbbs = NULL;
-  art = NULL;
   if(!(rbbs = rbbsopen(name, TRUE))){
     fprintf(stderr, "%s: \"%s\" could not open\n", g_progname, name);
     rv = 1;
@@ -529,6 +705,11 @@ int procdump(const char *name, int ftype){
       xprintf("<tagline>%@</tagline>\n", g_dumpsummary);
       xprintf("<generator>RBBS %@</generator>\n", RBBS_VERSION);
       break;
+    case FTLATEX:
+      printf("%s\n", latexpreamble("report", g_dumptitle, g_dumpauthor, rbbsmdate(rbbs)));
+      printf("\\begin{document}\n\n");
+      printf("\\maketitle\n\n");
+      break;
     default:
       xprintf("<?xml version=\"1.0\" encoding=\"%@\"?>\n", ENCODING);
       xprintf("<!DOCTYPE rbbs SYSTEM \"rbbs.dtd\">\n");
@@ -559,6 +740,11 @@ int procdump(const char *name, int ftype){
         printf("%s", xml);
         free(xml);
         break;
+      case FTLATEX:
+        xml = arttolatex(art, TRUE);
+        printf("%s", xml);
+        free(xml);
+        break;
       default:
         xml = arttoxml(art);
         printf("%s", xml);
@@ -580,6 +766,9 @@ int procdump(const char *name, int ftype){
     case FTATOM:
       xprintf("</feed>\n");
       break;
+    case FTLATEX:
+      printf("\\end{document}\n");
+      break;
     default:
       xprintf("</rbbs>\n");
       break;
@@ -593,6 +782,130 @@ int procdump(const char *name, int ftype){
 }
 
 
+/* save a file */
+int procsavefile(const char *name, const char *file, const char *id, const char *password,
+                 const char *subject, const char *author, const char *cdate){
+  RBBS *rbbs;
+  char *buf;
+  int rv, len, isdir;
+  rv = 0;
+  buf = NULL;
+  len = 0;
+  rbbs = NULL;
+  if(cbfilestat(name, &isdir, NULL, NULL) && !isdir){
+    fprintf(stderr, "%s: %s: not a database\n", g_progname, name);
+    rv = 1;
+  } else if(!(buf = readfiledata(file, &len))){
+    fprintf(stderr, "%s: %s: cannot read\n", g_progname, file ? file : "(stdin)");
+    rv = 1;
+  } else if(!(rbbs = rbbsopen(name, TRUE))){
+    fprintf(stderr, "%s: %s: cannot open\n", g_progname, name);
+    rv = 1;
+  } else {
+    if(!savefile(rbbs, id, password, subject, author, cdate, buf, len)){
+      fprintf(stderr, "%s: %s\n", g_progname, rbbsemsg(rbbs));
+      rv = 1;
+    } else {
+      printf("%s: %s: imported\n", name, file ? file : "(stdin)");
+    }
+  }
+  if(rbbs && !rbbsclose(rbbs)){
+    fprintf(stderr, "%s: %s: some errors occurred around the database\n", g_progname, name);
+    rv = 1;
+  }
+  if(buf) free(buf);
+  return rv;
+}
+
+
+/* load a file */
+int procloadfile(const char *name, const char *id, int al){
+  RBBS *rbbs;
+  CBMAP *attrs;
+  char *buf;
+  int i, rv, len;
+  rv = 0;
+  rbbs = NULL;
+  attrs = cbmapopen();
+  if(!(rbbs = rbbsopen(name, FALSE))){
+    fprintf(stderr, "%s: \"%s\" could not open\n", g_progname, name);
+    rv = 1;
+  } else if((buf = loadfile(rbbs, id, &len, attrs)) != NULL){
+    if(al){
+      printf("password: %s\n", cbmapget(attrs, "password", -1, NULL));
+      printf("subject: %s\n", cbmapget(attrs, "subject", -1, NULL));
+      printf("author: %s\n", cbmapget(attrs, "author", -1, NULL));
+      printf("cdate: %s\n", cbmapget(attrs, "cdate", -1, NULL));
+      printf("size: %s\n", cbmapget(attrs, "size", -1, NULL));
+    } else {
+      for(i = 0; i < len; i++){
+        putchar(buf[i]);
+      }
+    }
+    free(buf);
+  } else {
+    fprintf(stderr, "%s: %s\n", g_progname, rbbsemsg(rbbs));
+    rv = 1;
+  }
+  cbmapclose(attrs);
+  if(rbbs && !rbbsclose(rbbs)){
+    fprintf(stderr, "%s: %s: some errors occurred around the database\n", g_progname, name);
+    rv = 1;
+  }
+  return rv;
+}
+
+
+/* delete a file */
+int procdeletefile(const char *name, const char *id){
+  RBBS *rbbs;
+  int rv;
+  rv = 0;
+  rbbs = NULL;
+  if(!(rbbs = rbbsopen(name, TRUE))){
+    fprintf(stderr, "%s: \"%s\" could not open\n", g_progname, name);
+    rv = 1;
+  } else if(!deletefile(rbbs, id)){
+    fprintf(stderr, "%s: %s\n", g_progname, rbbsemsg(rbbs));
+    rv = 1;
+  } else {
+    printf("%s: %s: deleted\n", g_progname, id);
+  }
+  if(rbbs && !rbbsclose(rbbs)){
+    fprintf(stderr, "%s: %s: some errors occurred around the database\n", g_progname, name);
+    rv = 1;
+  }
+  return rv;
+}
+
+
+/* list files */
+int proclistfile(const char *name){
+  RBBS *rbbs;
+  CBLIST *list;
+  int i, rv;
+  rv = 0;
+  rbbs = NULL;
+  if(!(rbbs = rbbsopen(name, FALSE))){
+    fprintf(stderr, "%s: \"%s\" could not open\n", g_progname, name);
+    rv = 1;
+  } else if((list = listfiles(rbbs)) != NULL){
+    for(i = 0; i < cblistnum(list); i++){
+      printf("%s\n", cblistval(list, i, NULL));
+    }
+    cblistclose(list);
+  } else {
+    fprintf(stderr, "%s: %s\n", g_progname, rbbsemsg(rbbs));
+    rv = 1;
+  }
+  if(rbbs && !rbbsclose(rbbs)){
+    fprintf(stderr, "%s: %s: some errors occurred around the database\n", g_progname, name);
+    rv = 1;
+  }
+  return rv;
+}
+
+
 /* optimize a database */
 int procoptimize(const char *name){
   RBBS *rbbs;
@@ -654,17 +967,17 @@ int procpassword(const char *key, const char *code){
 
 
 /* read whole data of a file */
-char *readfiledata(const char *file){
+char *readfiledata(const char *file, int *sp){
   CBDATUM *buf;
   char cc;
   int c;
-  if(file) return cbreadfile(file, NULL);
+  if(file) return cbreadfile(file, sp);
   buf = cbdatumopen("", 0);
   while((c = getchar()) != EOF){
     cc = (char)c;
     cbdatumcat(buf, &cc, 1);
   }
-  return cbdatumtomalloc(buf, NULL);
+  return cbdatumtomalloc(buf, sp);
 }
 
 
